Mongoose Relationships
======================

_... because sometimes embedded documents aren't enough._

A plugin for [Mongoose](http://github.com/learnboost/mongoose) adding a simple syntax for model relationships and providing useful helpers to empower them.

This is an early release with limited functionalities. I'm looking for feedback on the API and features (been exploring a few different solutions, nothing's impossible!).

I'm inspiring from various libraries in Ruby I've used throughout the years. Might not be your cup of tea.

Goals
-----

* Be unobtrusive and compliant with the ways of Mongoose (coding style, testing, API).

Usage
=====

First, `npm install mongoose-relationships`.

Then, through mongoose's `Schema.plugin` method, you "attach" the plugin to your `Schema` as such:

```javascript
var relationships = require('mongoose-relationships');

YourSchema.plugin(relationships, {
    hasMany : "Model"  // Model name
  , through : "models" // Path where this is stored
});
```

Examples
--------

```javascript
// One to Many
UserSchema.plugin(relationships, {
    hasMany : "Post"
  , through : "posts"
});
PostSchema.plugin(relationships, {
    belongsTo : "User"
  , through   : "author"
});
```

More in the `examples` folder.

Methods
=======

Every `Document` that has their `Schema` plugged with `mongoose-relationships` has access to the following methods.

__Let's use this starting point:__

```javascript
// UserSchema stores an Array of ObjectIds for posts
var UserSchema = new mongoose.Schema({
    posts: [mongoose.Schema.ObjectId]
});

// PostSchema stores an ObjectId for the author
var PostSchema = new mongoose.Schema({
    title  : String
  , author : mongoose.Schema.ObjectId
});

// Attach the plugin
UserSchema.plugin(relationships, {
    hasMany: "Post"
  , through: "posts"
});
PostSchema.plugin(relationships, {
    belongsTo: "User"
  , through: "author"
});

var User = mongoose.model('User', UserSchema)
  , Post = mongoose.model('Post', PostSchema);
```

create
------

```javascript
var user = new User();

user.posts.create({
  title: "Mongoose, now with added love through relationships!"
}, function(err, user, post){
  // user.posts.length === 1
  // post.title === "Mongoose, now with added love through relationships!"
});

// Using an `Array`
user.posts.create([
    { title: "Not too imaginative post title" }
  , { title: "... a tad more imaginative post title" }
], function(err, user, posts){
  // user.posts.length === 3
  // posts.length == 2
  // posts[0] instanceof Post
});
```

find
----

It's the same as a `Mongoose.Query`. Only looks through the children documents.

```javascript
user.posts.find({title: "Not too imaginative post title"}, function(err, posts){
  // posts.length === 1
  // posts[0].author == user._id
  // posts[0].title == "Not too imaginative post title";
});
```

__That's it for now. I'm aware this isn't much. I'm already developing more as I type this.__

Testing
=======

I'm using [Expresso](http://github.com/visionmedia/expresso) w/ [Should](http://github.com/visionmedia/should.js). Tests are located in `./test` and should be ran with the `make test` command. 

Todo
====

* One to one relationships
* Instantiating new child documents without saving
* Better API

Contribute
==========

* Pick up any of the items above & send a pull request (w/ __passing__ tests please)
* Discuss the API / features in the [Issues](http://github.com/jeromegn/mongoose-relationships/issues)
* Use it and report bugs in the [Issues](http://github.com/jeromegn/mongoose-relationships/issues) (w/ __failing__ tests please)

